#include "sgfgame.h"
#include "sgfhelper.h"
#include "resource.h"
#include <stdio.h>

#define APP_WIN_NAME ("SGFWin")

const int DEFAULT_FPS = 60;
const int DEFAULT_WINDOW_WIDTH = 800;
const int DEFAULT_WINDOW_HEIGHT = 600;


//
static void renderClear(SystemModule* system)
{
	if (system) {
		SDL_SetRenderDrawColor(system->pRenderer, 0, 0, 0, 255);
		SDL_RenderClear(system->pRenderer);
	}
}

//FPS
static int renderFPS(SystemModule* system, float elapsed)
{
	static float _totalTime = 0;
	static int _fps = 0;
	static int _fpsCount = 0;

	_totalTime += elapsed;

	if (_totalTime >= 1000.0f)
	{
		_totalTime -= 1000.0f;
		_fps = _fpsCount;

		_fpsCount = 1;
	}
	else
	{
		++_fpsCount;
	}


	//ִС
	char fpsText[256] = { 0 };
	sprintf_s(fpsText, 255, "FPS: %d", _fps);
	SDL_Rect dst = {0, 0, 100, 21};
	int ret = TTF_SizeText(system->pDefFont, fpsText, &(dst.w), &(dst.h));
	if (ret) {
		SDL_Log("get text size failed: %s\n", TTF_GetError());
	}


	//Ⱦ
	SDL_Color color = { 255, 255, 255, 255 };
	if (system) {
		SDL_Surface* surf = TTF_RenderText_Blended(system->pDefFont, fpsText, color);
		if (surf) {
			SDL_Texture* texture = SDL_CreateTextureFromSurface(system->pRenderer, surf);
			if (texture) {
				SDL_RenderCopy(system->pRenderer, texture, NULL, &dst);
				SDL_DestroyTexture(texture);
			}
			SDL_FreeSurface(surf);
		}
	}

	return 0;
}


//ʼ
static void uninitGameModule(GameModule* pModule)
{
	SDL_assert(pModule != NULL);

	if (pModule && pModule->sys.pDefFont) {
		TTF_CloseFont(pModule->sys.pDefFont);
		pModule->sys.pDefFont = NULL;
	}

	if (pModule && pModule->sys.pRenderer) {
		SDL_DestroyRenderer(pModule->sys.pRenderer);
		pModule->sys.pRenderer = NULL;
	}

	if (pModule && pModule->sys.pWin) {
		SDL_DestroyWindow(pModule->sys.pWin);
		pModule->sys.pWin = NULL;
	}
	

	Mix_CloseAudio();
	Mix_Quit();
	IMG_Quit();
	TTF_Quit();
	SDL_Quit();

	if (pModule) {
		free(pModule);
	}
}


//ʼ
GameModule* initGameModule(HINSTANCE hinst)
{
	int ret = 0;

	//ģ
	GameModule* pModule = (GameModule*)malloc(sizeof(GameModule));
	if (!pModule) {
		SDL_Log("malloc unable to allocate memory\n");
		return NULL;
	}


	//Ĭֵ
	memset(pModule, 0, sizeof(GameModule));
	pModule->sys.hinstance = hinst;
	strcpy_s(pModule->usr.title, strlen(APP_WIN_NAME) + 1, APP_WIN_NAME);
	pModule->usr.winSize.cx = DEFAULT_WINDOW_WIDTH;
	pModule->usr.winSize.cy = DEFAULT_WINDOW_HEIGHT;
	pModule->usr.FPS = DEFAULT_FPS;
	pModule->usr.showFPS = 1;


	//ʼϵͳ
	ret = SDL_Init(SDL_INIT_EVERYTHING);
	if (ret != 0) {
		SDL_Log("Unable to initialize SDL: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//ʼ TTF
	ret = TTF_Init();
	if (ret != 0) {
		SDL_Log("Unable to initialize TTF: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//ʼ Image
	int flags = IMG_INIT_PNG | IMG_INIT_JPG | IMG_INIT_WEBP;
	int initted = IMG_Init(flags);
	if ((initted & flags) != flags) {
		SDL_Log("Failed to init required jpg and png and webp support: %s\n", IMG_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//ʼ mixer
	flags = MIX_INIT_FLAC | MIX_INIT_MOD | MIX_INIT_MP3 | MIX_INIT_OGG | MIX_INIT_MID | MIX_INIT_OPUS;
	initted = Mix_Init(flags);
	if ((initted & flags) != flags) {
		SDL_Log("Failed to init required flag and mod and mp3 and ogg and mid and opus support: %s\n", Mix_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//Ƶ
	ret = Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, MIX_DEFAULT_CHANNELS, 2048);
	if (ret != 0) {
		SDL_Log("Mix_OpenAudio failed: %s\n", Mix_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//ûʼ
	ret = initGame(&(pModule->usr));
	if (ret != 0) {
		uninitGameModule(pModule);
		return NULL;
	}


	//
	pModule->sys.pWin = SDL_CreateWindow(pModule->usr.title,
		SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
		pModule->usr.winSize.cx, pModule->usr.winSize.cy, 0);
	if (pModule->sys.pWin == NULL) {
		SDL_Log("Could not create window: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//ͼ
	SDL_RWops* rwOps = getResourceData(pModule->sys.hinstance, MAKEINTRESOURCE(IDI_ICON1), RT_ICON);
	if (rwOps) {
		SDL_Surface* iconSurface = IMG_Load_RW(rwOps, 1);
		if (iconSurface) {
			SDL_SetWindowIcon(pModule->sys.pWin, iconSurface);
			SDL_FreeSurface(iconSurface);
		}
	}


	//Ⱦ
	pModule->sys.pRenderer = SDL_CreateRenderer(pModule->sys.pWin, -1, SDL_RENDERER_ACCELERATED);
	if (pModule->sys.pRenderer == NULL) {
		SDL_Log("Could not create renderer: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//
	rwOps = getResourceData(pModule->sys.hinstance, MAKEINTRESOURCE(IDR_DEFAULT_FONT), RT_FONT);
	if (rwOps == NULL) {
		SDL_Log("Could not get font resource data: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}

	pModule->sys.pDefFont = TTF_OpenFontRW(rwOps, 1, 18);
	if (pModule->sys.pDefFont == NULL) {
		SDL_Log("Could not open font: %s\n", SDL_GetError());
		uninitGameModule(pModule);
		return NULL;
	}


	//Դ
	ret = loadGameResources(&(pModule->sys));
	if (ret != 0) {
		uninitGameModule(pModule);
		return NULL;
	}


	return pModule;
}


//
void runGame(GameModule* pModule)
{
	int fps = DEFAULT_FPS;
	if (pModule && pModule->usr.FPS > 0) {
		fps = pModule->usr.FPS;
	}

	int quit = 0;
	SDL_Event evt;

	Uint64 nFrequency, nPrevCounter, nCurrCounter, nElapsedCounter;
	float elapsed = 0.0f, lag = 0.0f, frameMS = 1000.0f / fps;


	nFrequency = SDL_GetPerformanceFrequency();
	nPrevCounter = SDL_GetPerformanceCounter();

	while (!quit) {
		if (SDL_PollEvent(&evt)) {
			if (evt.type == SDL_QUIT) {
				quit = 1;
			} else {
				processGameEvent(&evt);
			}
		} else {

			nCurrCounter = SDL_GetPerformanceCounter();
			nElapsedCounter = nCurrCounter - nPrevCounter;
			nPrevCounter = nCurrCounter;

			//ǰ֡ĺʱ(ms)
			elapsed = (nElapsedCounter * 1000.0f) / nFrequency;
			lag += elapsed;


			if (pModule) {
				//Ϸ
				while (lag >= frameMS) {
					updateGame(&(pModule->sys), frameMS);
					lag -= frameMS;
				}

				//
				renderClear(&(pModule->sys));

				//Ⱦͼ
				renderGame(&(pModule->sys), lag);

				//Ⱦ֡
				if (pModule && pModule->usr.showFPS) {
					renderFPS(&(pModule->sys), elapsed);
				}

				//ת
				SDL_RenderPresent(pModule->sys.pRenderer);
			}
		}
	}


	//
	if (pModule) {
		uninitGame(&(pModule->sys));
		uninitGameModule(pModule);
	}
}
